
import UIKit

let viewNodesMap = NodesMap()
var parentsMap = [Node: Set<Node>]()

protocol MacawContainer: AnyObject {
    var node: Node { get set }
}

class NodesMap {
    let map = NSMapTable<Node, AnyObject>(keyOptions: NSMapTableWeakMemory, valueOptions: NSMapTableWeakMemory)

	// MARK: - Macaw View
	func add(_ node: Node, container: MacawContainer) {
        map.setObject(container, forKey: node)

		if let group = node as? Group {
			group.contents.forEach { child in
				self.add(child, container: container)
				self.add(child, parent: node)
			}
		}
	}

	func getContainer(_ node: Node) -> MacawContainer? {
		return map.object(forKey: node) as? MacawContainer
	}

	func remove(_ node: Node) {
        map.removeObject(forKey: node)
		parentsMap.removeValue(forKey: node)
	}

	// MARK: - Parents
	func add(_ node: Node, parent: Node) {
		if var nodesSet = parentsMap[node] {
			nodesSet.insert(parent)
		} else {
			parentsMap[node] = Set([parent])
		}
        
        if let group = node as? Group {
            group.contents.forEach { child in
                self.add(child, parent: node)
            }
        }
	}

	func parents(_ node: Node) -> [Node] {
		guard let nodesSet = parentsMap[node] else {
			return []
		}

		return Array(nodesSet)
	}
    
    func replace(node: Node, to: Node) {
        let parents = parentsMap[node]
        let hostingContainer = map.object(forKey: node) as? MacawContainer
        
        remove(node)
        
        parents?.forEach { parent in
            guard let group = parent as? Group else {
                return
            }
            
            var contents = group.contents
            var indexToInsert = 0
            if let index = contents.index(of: node) {
                contents.remove(at: index)
                indexToInsert = index
            }
            
            contents.insert(to, at: indexToInsert)
            group.contents = contents
            
            add(to, parent: parent)
        }
        
        if let container = hostingContainer {
            add(to, container: container)
        }
        
        
        // Replacing node in hosting view if needed
        guard let hostingNode =  hostingContainer?.node else {
            return
        }
        
        if hostingNode == node {
            hostingContainer?.node = to
        }
    }
}
